@@ -8,9 +8,8 @@ import numpy as np
 import lxml.etree as ET
 import cv2
 
-from pero_ocr.ocr_engine.softmax import softmax
 from pero_ocr.document_ocr.crop_engine import EngineLineCropper
-from pero_ocr.force_alignment import force_align
+from pero_ocr.force_alignment import align_text
 
 
 def log_softmax(x):
@@ -19,7 +18,8 @@ def log_softmax(x):
 
 
 class TextLine(object):
-    def __init__(self, id=None, baseline=None, polygon=None, heights=None, transcription=None, logits=None, crop=None, characters=None):
+    def __init__(self, id=None, baseline=None, polygon=None, heights=None, transcription=None, logits=None, crop=None,
+                 characters=None):
         self.id = id
         self.baseline = baseline
         self.polygon = polygon
@@ -50,7 +50,7 @@ class RegionLayout(object):
         region_element = ET.SubElement(page_element, "TextRegion")
         coords = ET.SubElement(region_element, "Coords")
         region_element.set("id", self.id)
-        points = ["{},{}".format(int(coord[0]), int(coord[1])) for coord in self.polygon]
+        points = ["{},{}".format(int(np.round(coord[0])), int(np.round(coord[1]))) for coord in self.polygon]
         points = " ".join(points)
         coords.set("points", points)
         if self.transcription is not None:
@@ -168,13 +168,15 @@ class PageLayout(object):
                 coords = ET.SubElement(text_line, "Coords")
 
                 if line.polygon is not None:
-                    points = ["{},{}".format(int(coord[0]), int(coord[1])) for coord in line.polygon]
+                    points = ["{},{}".format(int(np.round(coord[0])), int(np.round(coord[1]))) for coord in
+                              line.polygon]
                     points = " ".join(points)
                     coords.set("points", points)
 
                 if line.baseline is not None:
                     baseline_element = ET.SubElement(text_line, "Baseline")
-                    points = ["{},{}".format(int(coord[0]), int(coord[1])) for coord in line.baseline]
+                    points = ["{},{}".format(int(np.round(coord[0])), int(np.round(coord[1]))) for coord in
+                              line.baseline]
                     points = " ".join(points)
                     baseline_element.set("points", points)
 
@@ -214,7 +216,7 @@ class PageLayout(object):
 
         layout = ET.SubElement(root, "Layout")
         page = ET.SubElement(layout, "Page")
-        page.set("ID", "id_"+self.id)
+        page.set("ID", "id_" + self.id)
         page.set("PHYSICAL_IMG_NR", str(1))
         page.set("HEIGHT", str(self.page_size[0]))
         page.set("WIDTH", str(self.page_size[1]))
@@ -229,24 +231,19 @@ class PageLayout(object):
         print_space_width = 0
         print_space_vpos = self.page_size[0]
         print_space_hpos = self.page_size[1]
+
         for b, block in enumerate(self.regions):
             text_block = ET.SubElement(print_space, "TextBlock")
-            text_block.set("ID", block.id)
+            text_block.set("ID", 'block_{}' .format(block.id))
 
-            text_block_height = max(block.polygon[:, 1]) - min(block.polygon[:, 1])
-            text_block.set("HEIGHT", str(text_block_height))
+            text_block_height, text_block_width, text_block_vpos, text_block_hpos = get_hwvh(block.polygon)
+            text_block.set("HEIGHT", str(int(text_block_height)))
+            text_block.set("WIDTH", str(int(text_block_width)))
+            text_block.set("VPOS", str(int(text_block_vpos)))
+            text_block.set("HPOS", str(int(text_block_hpos)))
 
-            text_block_width = max(block.polygon[:, 0]) - min(block.polygon[:, 0])
-            text_block.set("WIDTH", str(text_block_width))
-
-            text_block_vpos = min(block.polygon[:, 1])
-            text_block.set("VPOS", str(text_block_vpos))
-
-            text_block_hpos = min(block.polygon[:, 0])
-            text_block.set("HPOS", str(text_block_hpos))
-
-            print_space_height = max([print_space_vpos+print_space_height, text_block_vpos+text_block_height])
-            print_space_width = max([print_space_hpos+print_space_width, text_block_hpos+text_block_width])
+            print_space_height = max([print_space_vpos + print_space_height, text_block_vpos + text_block_height])
+            print_space_width = max([print_space_hpos + print_space_width, text_block_hpos + text_block_width])
             print_space_vpos = min([print_space_vpos, text_block_vpos])
             print_space_hpos = min([print_space_hpos, text_block_hpos])
             print_space_height = print_space_height - print_space_vpos
@@ -259,104 +256,109 @@ class PageLayout(object):
                 text_line_baseline = int(np.average(np.array(line.baseline)[:, 1]))
                 text_line.set("BASELINE", str(text_line_baseline))
 
-                text_line_vpos = min(np.array(line.polygon)[:, 1])
-                text_line.set("VPOS", str(text_line_vpos))
-                text_line_hpos = min(np.array(line.polygon)[:, 0])
-                text_line.set("HPOS", str(text_line_hpos))
-                text_line_height = max(np.array(line.polygon)[:, 1]) - min(np.array(line.polygon)[:, 1])
-                text_line.set("HEIGHT", str(text_line_height))
-                text_line_width = max(np.array(line.polygon)[:, 0]) - min(np.array(line.polygon)[:, 0])
-                text_line.set("WIDTH", str(text_line_width))
+                text_line_height, text_line_width, text_line_vpos, text_line_hpos = get_hwvh(line.polygon)
+
+                text_line.set("VPOS", str(int(text_line_vpos)))
+                text_line.set("HPOS", str(int(text_line_hpos)))
+                text_line.set("HEIGHT", str(int(text_line_height)))
+                text_line.set("WIDTH", str(int(text_line_width)))
 
                 chars = [i for i in range(len(line.characters))]
                 char_to_num = dict(zip(line.characters, chars))
 
+                blank_idx = line.logits.shape[1] - 1
+
                 label = []
-                for item in (line.transcription):
-                    label.append(char_to_num[item])
+                for item in line.transcription:
+                    if item in char_to_num.keys():
+                        if char_to_num[item] >= blank_idx:
+                            label.append(0)
+                        else:
+                            label.append(char_to_num[item])
+                    else:
+                        label.append(0)
 
                 logits = line.get_dense_logits()
-                output = softmax(logits, axis=1)
-                aligned = force_align(-np.log(output), label, len(chars))
-                narrow_label(aligned, logits, len(chars))
+                logprobs = line.get_full_logprobs()
+                try:
+                    aligned_letters = align_text(-logprobs, np.array(label), blank_idx)
+                except ValueError as _:
+                    average_word_width = (text_line_hpos + text_line_width) / len(line.transcription.split())
+                    for w, word in enumerate(line.transcription.split()):
+                        string = ET.SubElement(text_line, "String")
+                        string.set("CONTENT", word)
 
-                crop_engine = EngineLineCropper(poly=2)
-                line_coords = crop_engine.get_crop_inputs(line.baseline, line.heights, 16)
+                        string.set("HEIGHT", str(int(text_line_height)))
+                        string.set("WIDTH", str(int(average_word_width)))
+                        string.set("VPOS", str(int(text_line_vpos)))
+                        string.set("HPOS", str(int(text_line_hpos + (w * average_word_width))))
+                else:
+                    crop_engine = EngineLineCropper(poly=2)
+                    line_coords = crop_engine.get_crop_inputs(line.baseline, line.heights, 16)
+                    space_idxs = [pos for pos, char in enumerate(line.transcription) if char == ' ']
+                    word = []
+                    words = []
+                    counter = 0
+                    for i, elem in enumerate(aligned_letters):
+                        try:
+                            if i == space_idxs[counter]:
+                                if counter != (len(space_idxs) - 1):
+                                    counter += 1
+                                words.append(word)
+                                word = []
+                            else:
+                                word.append(elem)
+                        except IndexError as _:
+                            word.append(elem)
 
-                global_letter_counter = 0
-                for w, word in enumerate(line.transcription.split()):
-                    local_letter_counter = 0
-                    word_lenght = len(word)
-                    string_width = 0
-                    string_hpos = 0
-                    end_of_space = 0
-                    final = False
-                    last = True
+                    words.append(word)
 
-                    for a, ali in enumerate(aligned):
-                        if ali != len(chars):
-                            if local_letter_counter > global_letter_counter:
-                                if final:
-                                    end_of_space = 4*a
-                                    global_letter_counter = local_letter_counter
-                                    last = False
-                                    break
-                                if local_letter_counter - global_letter_counter == word_lenght:
-                                    string_width = 4*a - string_hpos
-                                    final = True
-                            elif local_letter_counter - global_letter_counter == 0:
-                                string_hpos = 4*a
-                            local_letter_counter += 1
+                    try:
+                        words.remove([])
+                    except ValueError:
+                        pass
 
-                    if last:
-                        string_width = 4 * len(aligned) - string_hpos
+                    start_end = []
+                    for i in words:
+                        start_end.append(tuple([i[0], i[-1]]))
 
-                    lm_const = np.shape(line_coords)[1]/(len(aligned)*4)
+                    splitted_transcription = line.transcription.split()
+                    lm_const = line_coords.shape[1] / logits.shape[0]
+                    for w, word in enumerate(start_end):
+                        string = ET.SubElement(text_line, "String")
+                        string.set("CONTENT", splitted_transcription[w])
+                        all_x = line_coords[:, int((start_end[w][0]-2) * lm_const):int((start_end[w][1]+2) * lm_const), 0]
+                        all_y = line_coords[:, int((start_end[w][0]-2) * lm_const):int((start_end[w][1]+2) * lm_const), 1]
 
-                    string = ET.SubElement(text_line, "String")
-                    string.set("CONTENT", word)
+                        string.set("HEIGHT", str(int((np.max(all_y) - np.min(all_y)))))
+                        string.set("WIDTH", str(int((np.max(all_x) - np.min(all_x)))))
+                        string.set("VPOS", str(int(np.min(all_y))))
+                        string.set("HPOS", str(int(np.min(all_x))))
 
-                    string_hpos -= 1
-                    all_x = line_coords[:, int(string_hpos*lm_const):int(string_hpos*lm_const)+int(string_width*lm_const), 0]
-                    all_y = line_coords[:, int(string_hpos*lm_const):int(string_hpos*lm_const)+int(string_width*lm_const), 1]
-
-                    string.set("HEIGHT", str(int(np.max(all_y)-np.min(all_y))))
-                    string.set("WIDTH", str(int(np.max(all_x)-np.min(all_x))))
-                    string.set("VPOS", str(int(np.min(all_y))))
-                    string.set("HPOS", str(int(np.min(all_x))))
-                    if w != (len(line.transcription.split())-1):
-                        space = ET.SubElement(text_line, "SP")
-                        all_x = line_coords[:, int((string_hpos+string_width) * lm_const):int((string_hpos+string_width) * lm_const) + int((end_of_space-(string_hpos+string_width)) * lm_const), 0]
-                        all_y = line_coords[:, int((string_hpos+string_width) * lm_const):int((string_hpos+string_width) * lm_const) + int((end_of_space-(string_hpos+string_width)) * lm_const), 1]
-
-                        space.set("WIDTH", str(int(np.max(all_x)-np.min(all_x))))
-                        space.set("VPOS", str(int(np.min(all_y))))
-                        space.set("HPOS", str(int(np.min(all_x))))
-
-        top_margin.set("HEIGHT", "{}" .format(print_space_vpos))
-        top_margin.set("WIDTH", "{}" .format(self.page_size[1]))
+        top_margin.set("HEIGHT", "{}" .format(int(print_space_vpos)))
+        top_margin.set("WIDTH", "{}" .format(int(self.page_size[1])))
         top_margin.set("VPOS", "0")
         top_margin.set("HPOS", "0")
 
-        left_margin.set("HEIGHT", "{}" .format(self.page_size[0]))
-        left_margin.set("WIDTH", "{}" .format(print_space_hpos))
+        left_margin.set("HEIGHT", "{}" .format(int(self.page_size[0])))
+        left_margin.set("WIDTH", "{}" .format(int(print_space_hpos)))
         left_margin.set("VPOS", "0")
         left_margin.set("HPOS", "0")
 
-        right_margin.set("HEIGHT", "{}" .format(self.page_size[0]))
-        right_margin.set("WIDTH", "{}" .format(self.page_size[1]-(print_space_hpos+print_space_width)))
+        right_margin.set("HEIGHT", "{}" .format(int(self.page_size[0])))
+        right_margin.set("WIDTH", "{}" .format(int(self.page_size[1] - (print_space_hpos + print_space_width))))
         right_margin.set("VPOS", "0")
-        right_margin.set("HPOS", "{}" .format(print_space_hpos+print_space_width))
+        right_margin.set("HPOS", "{}" .format(int(print_space_hpos + print_space_width)))
 
-        bottom_margin.set("HEIGHT", "{}" .format(self.page_size[0]-(print_space_vpos+print_space_height)))
-        bottom_margin.set("WIDTH", "{}" .format(self.page_size[1]))
-        bottom_margin.set("VPOS", "{}" .format(print_space_vpos+print_space_height))
+        bottom_margin.set("HEIGHT", "{}" .format(int(self.page_size[0] - (print_space_vpos + print_space_height))))
+        bottom_margin.set("WIDTH", "{}" .format(int(self.page_size[1])))
+        bottom_margin.set("VPOS", "{}" .format(int(print_space_vpos + print_space_height)))
         bottom_margin.set("HPOS", "0")
 
-        print_space.set("HEIGHT", str(print_space_height))
-        print_space.set("WIDTH", str(print_space_width))
-        print_space.set("VPOS", str(print_space_vpos))
-        print_space.set("HPOS", str(print_space_hpos))
+        print_space.set("HEIGHT", str(int(print_space_height)))
+        print_space.set("WIDTH", str(int(print_space_width)))
+        print_space.set("VPOS", str(int(print_space_vpos)))
+        print_space.set("HPOS", str(int(print_space_hpos)))
 
         return ET.tostring(root, pretty_print=True, encoding="utf-8").decode("utf-8")
 
@@ -373,29 +375,37 @@ class PageLayout(object):
         schema = element_schema(page_tree.getroot())
         root = page_tree.getroot()
 
-        layout = root.findall(schema+'Layout')[0]
-        page = layout.findall(schema+'Page')[0]
+        layout = root.findall(schema + 'Layout')[0]
+        page = layout.findall(schema + 'Page')[0]
 
         self.id = page.attrib['ID'][3:]
         self.page_size = (int(page.attrib['HEIGHT']), int(page.attrib['WIDTH']))
 
-        print_space = page.findall(schema+'PrintSpace')[0]
+        print_space = page.findall(schema + 'PrintSpace')[0]
         for region in print_space.iter(schema + 'TextBlock'):
             region_coords = list()
             region_coords.append([int(region.get('HPOS')), int(region.get('VPOS'))])
-            region_coords.append([int(region.get('HPOS'))+int(region.get('WIDTH')), int(region.get('VPOS'))])
-            region_coords.append([int(region.get('HPOS'))+int(region.get('WIDTH')), int(region.get('VPOS'))+int(region.get('HEIGHT'))])
-            region_coords.append([int(region.get('HPOS')), int(region.get('VPOS'))+int(region.get('HEIGHT'))])
+            region_coords.append([int(region.get('HPOS')) + int(region.get('WIDTH')), int(region.get('VPOS'))])
+            region_coords.append([int(region.get('HPOS')) + int(region.get('WIDTH')),
+                                  int(region.get('VPOS')) + int(region.get('HEIGHT'))])
+            region_coords.append([int(region.get('HPOS')), int(region.get('VPOS')) + int(region.get('HEIGHT'))])
 
             region_layout = RegionLayout(region.attrib['ID'], np.asarray(region_coords))
 
             for line in region.iter(schema + 'TextLine'):
-                new_textline = TextLine(baseline=[[int(line.attrib['HPOS']), int(line.attrib['BASELINE'])], [int(line.attrib['HPOS']) + int(line.attrib['WIDTH']), int(line.attrib['BASELINE'])]], polygon=[])
-                new_textline.heights = [int(line.attrib['HEIGHT'])+int(line.attrib['VPOS'])-int(line.attrib['BASELINE']), int(line.attrib['BASELINE'])-int(line.attrib['VPOS'])]
+                new_textline = TextLine(baseline=[[int(line.attrib['HPOS']), int(line.attrib['BASELINE'])],
+                                                  [int(line.attrib['HPOS']) + int(line.attrib['WIDTH']),
+                                                   int(line.attrib['BASELINE'])]], polygon=[])
+                new_textline.heights = [
+                    int(line.attrib['HEIGHT']) + int(line.attrib['VPOS']) - int(line.attrib['BASELINE']),
+                    int(line.attrib['BASELINE']) - int(line.attrib['VPOS'])]
                 new_textline.polygon.append([int(line.attrib['HPOS']), int(line.attrib['VPOS'])])
-                new_textline.polygon.append([int(line.attrib['HPOS'])+int(line.attrib['WIDTH']), int(line.attrib['VPOS'])])
-                new_textline.polygon.append([int(line.attrib['HPOS'])+int(line.attrib['WIDTH']), int(line.attrib['VPOS'])+int(line.attrib['HEIGHT'])])
-                new_textline.polygon.append([int(line.attrib['HPOS']), int(line.attrib['VPOS'])+int(line.attrib['HEIGHT'])])
+                new_textline.polygon.append(
+                    [int(line.attrib['HPOS']) + int(line.attrib['WIDTH']), int(line.attrib['VPOS'])])
+                new_textline.polygon.append([int(line.attrib['HPOS']) + int(line.attrib['WIDTH']),
+                                             int(line.attrib['VPOS']) + int(line.attrib['HEIGHT'])])
+                new_textline.polygon.append(
+                    [int(line.attrib['HPOS']), int(line.attrib['VPOS']) + int(line.attrib['HEIGHT'])])
                 word = ''
                 start = True
                 for text in line.iter(schema + 'String'):
@@ -447,20 +457,40 @@ class PageLayout(object):
                 line.logits = logits_dict[line.id]
                 line.characters = characters[line.id]
 
-    def render_to_image(self, image, thickness=2, circles=True):
+    def render_to_image(self, image, thickness=2, circles=True, render_order=False):
         """Render layout into image.
         :param image: image to render layout into
         """
         for region_layout in self.regions:
             image = draw_lines(
                 image,
-                [line.baseline for line in region_layout.lines if line.baseline is not None], color=(0,0,255), circles=(circles, circles, False), thickness=thickness)
+                [line.baseline for line in region_layout.lines if line.baseline is not None], color=(0, 0, 255),
+                circles=(circles, circles, False), thickness=thickness)
             image = draw_lines(
                 image,
-                [line.polygon for line in region_layout.lines if line.polygon is not None], color=(0,255,0), close=True, thickness=thickness)
+                [line.polygon for line in region_layout.lines if line.polygon is not None], color=(0, 255, 0),
+                close=True, thickness=thickness)
             image = draw_lines(
                 image,
-                [region_layout.polygon], color=(255, 0, 0), circles=(circles, circles, circles), close=True, thickness=thickness)
+                [region_layout.polygon], color=(255, 0, 0), circles=(circles, circles, circles), close=True,
+                thickness=thickness)
+
+        if render_order:
+            font = cv2.FONT_HERSHEY_DUPLEX
+            font_scale = 4
+            font_thickness = 5
+
+            for idx, region in enumerate(self.regions):
+                min = region.polygon.min(axis=0)
+                max = region.polygon.max(axis=0)
+
+                text_w, text_h = cv2.getTextSize(f"{idx}", font, font_scale, font_thickness)[0]
+
+                mid_coords = (int((min[0] + max[0]) // 2 - text_w // 2), int((min[1] + max[1]) // 2 + text_h // 2))
+
+                cv2.putText(image, f"{idx}", mid_coords, font, font_scale,
+                            (0, 0, 0), thickness=font_thickness, lineType=cv2.LINE_AA)
+
         return image
 
     def lines_iterator(self):
@@ -469,7 +499,7 @@ class PageLayout(object):
                 yield l
 
 
-def draw_lines(img, lines, color=(255,0,0), circles=(False, False, False), close=False, thickness=2):
+def draw_lines(img, lines, color=(255, 0, 0), circles=(False, False, False), close=False, thickness=2):
     """Draw a line into image.
     :param img: input image
     :param lines: list of arrays of line coords
@@ -481,16 +511,18 @@ def draw_lines(img, lines, color=(255,0,0), circles=(False, False, False), close
         first = line[0]
         last = first
         if circles[0]:
-            cv2.circle(img, (int(last[0]), int(last[1])), 3, color, 4)
+            cv2.circle(img, (int(np.round(last[0])), int(np.round(last[1]))), 3, color, 4)
         for p in line[1:]:
-            cv2.line(img, (int(last[0]), int(last[1])), (int(p[0]), int(p[1])), color, thickness)
+            cv2.line(img, (int(np.round(last[0])), int(np.round(last[1]))), (int(np.round(p[0])), int(np.round(p[1]))),
+                     color, thickness)
             if circles[1]:
-                cv2.circle(img, (int(last[0]), int(last[1])), 3, color, 4)
+                cv2.circle(img, (int(np.round(last[0])), int(np.round(last[1]))), 3, color, 4)
             last = p
         if circles[1]:
-            cv2.circle(img, (int(line[-1][0]), int(line[-1][1])), 3, color, 4)
+            cv2.circle(img, (int(np.round(line[-1][0])), int(np.round(line[-1][1]))), 3, color, 4)
         if close:
-            cv2.line(img, (int(last[0]), int(last[1])), (int(first[0]), int(first[1])), color, thickness)
+            cv2.line(img, (int(np.round(last[0])), int(np.round(last[1]))),
+                     (int(np.round(first[0])), int(np.round(first[1]))), color, thickness)
     return img
 
 
@@ -520,39 +552,26 @@ def find_optimal(logit, positions, idx):
     return highest
 
 
-def narrow_label(label, logit, idx_of_last, on_one_liberal=False):
-    last_char = None
-    repeating = []
-    for i, item in enumerate(label):
-        if last_char == item and last_char != idx_of_last:
-            repeating.extend([i])
-        else:
-            if repeating != []:
-                high = repeating[0]
-                for e, elem in enumerate(repeating):
-                    if on_one_liberal:
-                        label[elem] = idx_of_last - 1
-                    else:
-                        label[elem] = idx_of_last
-                label[high] = last_char
-        if last_char != item:
-            repeating = []
-            if item != idx_of_last:
-                repeating.append(i)
-        last_char = item
-    if repeating != []:
-        high = repeating[0]
-        for i, item in enumerate(repeating):
-            if on_one_liberal:
-                label[item] = idx_of_last - 1
-            else:
-                label[item] = idx_of_last
-        label[high] = last_char
+def get_hwvh(polygon):
+    xy = list(zip(*polygon))
 
-    return label
+    height = max(xy[1]) - min(xy[1])
+    width = max(xy[0]) - min(xy[0])
+
+    vpos = min(xy[1])
+    hpos = min(xy[0])
+
+    return height, width, vpos, hpos
 
 
 if __name__ == '__main__':
+    l = PageLayout(
+        file='/home/ikohut/data/pero_ocr_web_data/ocr_client/0fb06b7c-92b3-41cd-9523-5a869dccd7dc/output/page/9baa3b0d-3a6c-41b9-86b3-a012ea0ed378.xml')
+    l.load_logits(
+        '/home/ikohut/data/pero_ocr_web_data/ocr_client/0fb06b7c-92b3-41cd-9523-5a869dccd7dc/output/logits/9baa3b0d-3a6c-41b9-86b3-a012ea0ed378.logits')
+    print(l.to_altoxml_string())
+
+
     # test_layout = PageLayout(file='/mnt/matylda1/ikodym/junk/refactor_test/8e41ecc2-57ed-412a-aa4f-d945efa7c624_gt.xml')
     # test_layout.to_pagexml('/mnt/matylda1/ikodym/junk/refactor_test/test.xml')
     # image = cv2.imread('/mnt/matylda1/ikodym/junk/refactor_test/8e41ecc2-57ed-412a-aa4f-d945efa7c624.jpg')
@@ -561,18 +580,8 @@ if __name__ == '__main__':
 
     def save():
         test_layout = PageLayout()
-        test_layout.from_pagexml('C:/Users/LachubCz_NTB/Documents/GitHub/pero-ocr/0580ee04-14d4-4142-a14e-d5e1b31f83e0.xml')
-        test_layout.load_logits('C:/Users/LachubCz_NTB/Documents/GitHub/pero-ocr/0580ee04-14d4-4142-a14e-d5e1b31f83e0.logits')
-
-        image = cv2.imread("C:/Users/LachubCz_NTB/Documents/GitHub/pero-ocr/0580ee04-14d4-4142-a14e-d5e1b31f83e0.jpeg")
-        cv2.imwrite("C:/Users/LachubCz_NTB/Documents/GitHub/pero-ocr/xml_page.jpg", test_layout.render_to_image(image))
+        test_layout.from_pagexml('C:/Users/LachubCz_NTB/Documents/GitHub/pero-ocr/00cfab43-a5bc-4af0-b1c4-b26925679afd.xml')
+        test_layout.load_logits('C:/Users/LachubCz_NTB/Documents/GitHub/pero-ocr/00cfab43-a5bc-4af0-b1c4-b26925679afd.logits')
         test_layout.to_altoxml("C:/Users/LachubCz_NTB/Documents/GitHub/pero-ocr/test_alto.xml")
 
-    def load():
-        test_layout = PageLayout()
-        test_layout.from_altoxml('C:/Users/LachubCz_NTB/Documents/GitHub/pero-ocr/test_alto.xml')
-        image = cv2.imread("C:/Users/LachubCz_NTB/Documents/GitHub/pero-ocr/0580ee04-14d4-4142-a14e-d5e1b31f83e0.jpeg")
-        cv2.imwrite("C:/Users/LachubCz_NTB/Documents/GitHub/pero-ocr/xml_alto.jpg", test_layout.render_to_image(image))
-
     save()
-    load()
