@@ -1,16 +1,18 @@
 import numpy as np
-from os.path import isabs, join, realpath
 
 from multiprocessing import Pool
 from functools import partial
 from scipy import ndimage
 
+from pero_ocr.utils import compose_path
+
 from .layout import PageLayout, RegionLayout, TextLine
 from pero_ocr.document_ocr import crop_engine as cropper
 from pero_ocr.ocr_engine import line_ocr_engine
 from pero_ocr.line_engine import baseline_engine
 from pero_ocr.region_engine import region_engine
 from pero_ocr.region_engine import region_engine_splic
+from pero_ocr.region_engine import region_engine_simple
 from pero_ocr.region_engine import SimpleThresholdRegion
 import pero_ocr.line_engine.line_postprocessing as linepp
 
@@ -25,6 +27,8 @@ def layout_parser_factory(config, config_path=''):
         region_parser = SimpleThresholdRegion(config, config_path=config_path)
     elif config['METHOD'] == 'SPLIC':
         region_parser = RegionExtractorSPLIC(config, config_path=config_path)
+    elif config['METHOD'] == 'SIMPLE':
+        region_parser = RegionExtractorSimple(config, config_path=config_path)
     else:
         raise ValueError('Unknown layout parser method: {}'.format(config['METHOD']))
     return region_parser
@@ -34,6 +38,8 @@ def line_parser_factory(config, config_path=''):
     config = config['LINE_PARSER']
     if config['METHOD'] == 'cnn':
         line_parser = TextlineExtractorCNN(config, config_path=config_path)
+    elif config['METHOD'] == 'cnn_reg':
+        line_parser = TextlineExtractorCNNReg(config, config_path=config_path)
     elif config['METHOD'] == 'simple':
         line_parser = TextlineExtractorSimple(config, config_path=config_path)
     elif config['METHOD'] == 'line_refiner':
@@ -55,15 +61,23 @@ def ocr_factory(config, config_path=''):
 
 def page_decoder_factory(config, config_path=''):
     from pero_ocr.decoding import decoding_itf
-    ocr_chars = decoding_itf.get_ocr_charset(config['OCR']['OCR_JSON'])
-    decoder = decoding_itf.decoder_factory(config['DECODER'], ocr_chars, allow_no_decoder=False)
-    return PageDecoder(decoder, config_path=config_path)
+    ocr_chars = decoding_itf.get_ocr_charset(compose_path(config['OCR']['OCR_JSON'], config_path))
+    decoder = decoding_itf.decoder_factory(config['DECODER'], ocr_chars, allow_no_decoder=False, use_gpu=True,
+                                           config_path=config_path)
+    return PageDecoder(decoder)
 
 
-def compose_path(file_path, reference_path):
-    if reference_path and not isabs(file_path):
-        file_path = join(reference_path, file_path)
-    return file_path
+def region_sorter_factory(config, config_path=''):
+    class_name = config['REGION_SORTER']['METHOD']
+    imported = __import__('pero_ocr.region_sorter', fromlist=[class_name])
+
+    try:
+        object_class = getattr(imported, class_name)
+    except AttributeError:
+        raise ValueError(f"Region sorter method not found. Ensure \"{class_name}\" is valid region sorter"
+                         " class name imported in \"__init__.py\".")
+
+    return object_class(config['REGION_SORTER'], config_path)
 
 
 class MissingLogits(Exception):
@@ -71,13 +85,20 @@ class MissingLogits(Exception):
 
 
 class PageDecoder:
-    def __init__(self, decoder):
+    def __init__(self, decoder, line_confidence_threshold=None):
         self.decoder = decoder
+        self.line_confidence_threshold = line_confidence_threshold
 
     def process_page(self, page_layout: PageLayout):
         for line in page_layout.lines_iterator():
             logits = self.prepare_dense_logits(line)
-            line.transcription = self.decoder(logits).best_hyp()
+            if self.line_confidence_threshold is not None:
+                if self.line_confident_enough(logits, self.line_confidence_threshold):
+                    continue
+
+            hypotheses = self.decoder(logits)
+            if hypotheses is not None:
+                line.transcription = hypotheses.best_hyp()
 
         return page_layout
 
@@ -87,6 +108,13 @@ class PageDecoder:
 
         return line.get_dense_logits()
 
+    def line_confident_enough(self, logits):
+        log_probs = logits - np.logaddexp.reduce(logits, axis=1)[:, np.newaxis]
+        best_probs = np.max(log_probs, axis=-1)
+        worst_best_prob = np.exp(np.min(best_probs))
+
+        return worst_best_prob > self.line_confidence_threshold
+
 
 class WholePageRegion(object):
     def __init__(self, config, config_path=''):
@@ -153,6 +181,39 @@ class RegionExtractorSPLIC(object):
 
         return page_layout
 
+class RegionExtractorSimple(object):
+    def __init__(self, config, config_path=''):
+        model_path = compose_path(config['MODEL_PATH'], config_path)
+        downsample = config.getint('DOWNSAMPLE')
+        use_cpu = config.getboolean('USE_CPU')
+        max_mp = config.getfloat('MAX_MEGAPIXELS')
+        gpu_fraction = config.getfloat('GPU_FRACTION')
+        self.keep_lines = config.getboolean('KEEP_LINES')
+        self.region_engine = region_engine_simple.EngineRegionSimple(
+            model_path=model_path,
+            downsample=downsample,
+            use_cpu=use_cpu,
+            max_mp=max_mp,
+            gpu_fraction=gpu_fraction
+        )
+        self.pool = Pool()
+
+    def process_page(self, img, page_layout: PageLayout):
+        polygons_list, baselines_list, heights_list, textlines_list = self.region_engine.detect(img)
+        for id, polygon in enumerate(polygons_list):
+            region = RegionLayout('r{:03d}'.format(id), polygon)
+            page_layout.regions.append(region)
+
+        if self.keep_lines:
+            if len(page_layout.regions) > 4:
+                page_layout.regions = list(self.pool.map(partial(assign_lines_to_region, baselines_list, heights_list, textlines_list),
+                                 page_layout.regions))
+            else:
+                for region in page_layout.regions:
+                    region = assign_lines_to_region(baselines_list, heights_list, textlines_list, region)
+
+        return page_layout
+
 
 class LineRefiner(object):
     def __init__(self, config, config_path=''):
@@ -160,7 +221,7 @@ class LineRefiner(object):
         self.downsample = config.getint('DOWNSAMPLE')
         self.pad = config.getint('PAD')
         self.use_cpu = config.getboolean('USE_CPU')
-        self.line_engine = baseline_engine.EngineLineDetectorCNN(
+        self.line_engine = baseline_engine.EngineLineDetectorCNNReg(
             model_path=self.model_path,
             downsample=self.downsample,
             pad=self.pad,
@@ -175,7 +236,8 @@ class LineRefiner(object):
             print(f"Warning: Skipping line reninement for page {page_layout.id}. No text lines present.")
             return page_layout
 
-        baselines_map, heights_map = self.line_engine.infer_maps(img)
+        out_map = self.line_engine.get_maps(img, self.downsample)
+        baseline_map, heights_map = out_map[:,:,2], out_map[:,:,:2]
 
         if self.adjust_baselines:
             baselines = [line.baseline for line in page_layout.lines_iterator()]
@@ -188,9 +250,11 @@ class LineRefiner(object):
             for line in page_layout.lines_iterator():
                 baseline = line.baseline / self.downsample
                 sample_points = linepp.resample_baselines([baseline], num_points=40)[0]
+                y_inds = np.clip(np.round(sample_points[:,1]).astype(np.int), 0, heights_map.shape[0]-1)
+                x_inds = np.clip(np.round(sample_points[:,0]).astype(np.int), 0, heights_map.shape[1]-1)
                 heights_pred = self.line_engine.get_heights(
                     heights_map,
-                    (np.round(sample_points[:,1]).astype(np.int), np.round(sample_points[:,0]).astype(np.int)))
+                    (y_inds, x_inds))
                 line.heights = heights_pred * self.downsample
 
         height = np.median([l.heights[0] + l.heights[1] for l in page_layout.lines_iterator()])
@@ -198,25 +262,26 @@ class LineRefiner(object):
             temp_downsample = self.downsample
             print("ADAPT DOWNAMPLING", img.shape[0:2], self.downsample, height, height / self.downsample)
             self.downsample = max(1, int(height / 12 + 0.5))
+            ds_threshold = (img.shape[0] * img.shape[1]) / (5 * 10e5)
+            if self.downsample < ds_threshold:
+                self.downsample = ds_threshold
 
             self.line_engine.downsample = self.downsample
-            baselines_map, heights_map = self.line_engine.infer_maps(img)
+            out_map = self.line_engine.get_maps(img, self.downsample)
+            baseline_map, heights_map = out_map[:,:,2], out_map[:,:,:2]
             self.line_engine.downsample = temp_downsample
             if self.adjust_heights:
                 heights_map = ndimage.morphology.grey_dilation(heights_map, size=(11, 1, 1))
                 for line in page_layout.lines_iterator():
                     baseline = line.baseline / self.downsample
                     sample_points = linepp.resample_baselines([baseline], num_points=40)[0]
+                    y_inds = np.clip(np.round(sample_points[:,1]).astype(np.int), 0, heights_map.shape[0]-1)
+                    x_inds = np.clip(np.round(sample_points[:,0]).astype(np.int), 0, heights_map.shape[1]-1)
                     heights_pred = self.line_engine.get_heights(
                         heights_map,
-                        (np.round(sample_points[:, 1]).astype(np.int), np.round(sample_points[:, 0]).astype(np.int)))
+                        (y_inds, x_inds))
                     line.heights = heights_pred * self.downsample
 
-            height = np.median([l.heights[0] + l.heights[1] for l in page_layout.lines_iterator()])
-            print(f"OPTIMAL DOWNAMPLING {img.shape[0] // self.downsample}:{img.shape[1] // self.downsample}",
-                  self.downsample, height, height / self.downsample)
-            self.downsample = temp_downsample
-
 
         for line in page_layout.lines_iterator():
             line.polygon = linepp.baseline_to_textline(line.baseline, line.heights)
@@ -337,6 +402,26 @@ class TextlineExtractorCNN(BaseTextlineExtractor):
             detection_threshold=detection_threshold,
         )
 
+class TextlineExtractorCNNReg(BaseTextlineExtractor):
+    def __init__(self, config, config_path=''):
+        super(TextlineExtractorCNNReg, self).__init__(config)
+        model_path = compose_path(config['MODEL_PATH'], config_path)
+        downsample = config.getint('DOWNSAMPLE')
+        pad = config.getint('PAD')
+        use_cpu = config.getboolean('USE_CPU')
+        detection_threshold = config.getfloat('DETECTION_THRESHOLD')
+        max_mp = config.getfloat('MAX_MEGAPIXELS')
+        gpu_fraction = config.getfloat('GPU_FRACTION')
+        self.line_engine = baseline_engine.EngineLineDetectorCNNReg(
+            model_path=model_path,
+            downsample=downsample,
+            pad=pad,
+            use_cpu=use_cpu,
+            detection_threshold=detection_threshold,
+            max_mp=max_mp,
+            gpu_fraction=gpu_fraction
+        )
+
 
 class TextlineExtractorSimple(object):
     def __init__(self, config, config_path=''):
@@ -400,21 +485,45 @@ class PageOCR(object):
         return page_layout
 
 
-class PageParser (object):
+def get_prob(best_ids, best_probs):
+    last_id = -1
+    last_prob = 1
+    worst_prob = 1
+    for id, prob in zip(best_ids, best_probs):
+        if id != last_id:
+            worst_prob = min(worst_prob, last_prob)
+            last_prob = prob
+            last_id = id
+        else:
+            last_prob = max(prob, last_prob)
+
+    worst_prob = min(worst_prob, last_prob)
+    return worst_prob
+
+
+class PageParser(object):
     def __init__(self, config, config_path=''):
-        self.run_layout_parser = config['PAGE_PARSER'].getboolean('RUN_LAYOUT_PARSER')
-        self.run_line_parser = config['PAGE_PARSER'].getboolean('RUN_LINE_PARSER')
-        self.run_line_cropper = config['PAGE_PARSER'].getboolean('RUN_LINE_CROPPER')
-        self.run_ocr = config['PAGE_PARSER'].getboolean('RUN_OCR')
-        self.run_decoder = config['PAGE_PARSER'].getboolean('RUN_DECODER')
+        self.run_layout_parser = config['PAGE_PARSER'].getboolean('RUN_LAYOUT_PARSER', fallback=False)
+        self.run_line_parser = config['PAGE_PARSER'].getboolean('RUN_LINE_PARSER', fallback=False)
+        self.run_line_cropper = config['PAGE_PARSER'].getboolean('RUN_LINE_CROPPER', fallback=False)
+        self.run_ocr = config['PAGE_PARSER'].getboolean('RUN_OCR', fallback=False)
+        self.run_decoder = config['PAGE_PARSER'].getboolean('RUN_DECODER', fallback=False)
+        self.run_region_sorter = config['PAGE_PARSER'].getboolean('RUN_REGION_SORTER', fallback=False)
+        self.filter_confident_lines_threshold = config['PAGE_PARSER'].getfloat('FILTER_CONFIDENT_LINES_THRESHOLD',
+                                                                                 fallback=-1)
 
         self.layout_parser = None
         self.line_parser = None
         self.line_cropper = None
         self.ocr = None
         self.decoder = None
+        self.region_sorter = None
+
         if self.run_layout_parser:
             self.layout_parser = layout_parser_factory(config, config_path=config_path)
+        if self.run_region_sorter:
+            self.region_sorter = region_sorter_factory(config, config_path=config_path)
+
         if self.run_line_parser:
             self.line_parser = line_parser_factory(config, config_path=config_path)
         if self.run_line_cropper:
@@ -424,9 +533,30 @@ class PageParser (object):
         if self.run_decoder:
             self.decoder = page_decoder_factory(config, config_path=config_path)
 
+    @staticmethod
+    def compute_line_confidence(line, threshold):
+        logits = line.get_dense_logits()
+        log_probs = logits - np.logaddexp.reduce(logits, axis=1)[:, np.newaxis]
+        best_ids = np.argmax(log_probs, axis=-1)
+        best_probs = np.exp(np.max(log_probs, axis=-1))
+        worst_best_prob = get_prob(best_ids, best_probs)
+        print(worst_best_prob, np.sum(np.exp(best_probs) < threshold), best_probs.shape,  np.nonzero(np.exp(best_probs) < threshold))
+        #for i in np.nonzero(np.exp(best_probs) < threshold)[0]:
+        #    print(best_probs[i-1:i+2], best_ids[i-1:i+2])
+
+        return worst_best_prob
+
+    def filter_confident_lines(self, page_layout):
+        for region in page_layout.regions:
+            region.lines = [line for line in region.lines
+                            if PageParser.compute_line_confidence(line, self.filter_confident_lines_threshold) > self.filter_confident_lines_threshold]
+        return page_layout
+
     def process_page(self, image, page_layout):
         if self.run_layout_parser:
             page_layout = self.layout_parser.process_page(image, page_layout)
+        if self.run_region_sorter:
+            page_layout = self.region_sorter.process_page(image, page_layout)
         if self.run_line_parser:
             page_layout = self.line_parser.process_page(image, page_layout)
         if self.run_line_cropper:
@@ -435,5 +565,7 @@ class PageParser (object):
             page_layout = self.ocr.process_page(image, page_layout)
         if self.run_decoder:
             page_layout = self.decoder.process_page(page_layout)
+        if self.filter_confident_lines_threshold > 0:
+            page_layout = self.filter_confident_lines(page_layout)
 
         return page_layout
